@@ -45,7 +45,7 @@
 static unsigned int sizeof_device_field[device_field_count] = {
     [device_name] = 11,       [device_fan_speed] = 11,   [device_temperature] = 10, [device_power] = 15,
     [device_clock] = 11,      [device_mem_clock] = 12,   [device_pcie] = 46,        [device_shadercores] = 7,
-    [device_l2features] = 11, [device_execengines] = 11, [device_effective_load] = 12,
+    [device_l2features] = 11, [device_execengines] = 11,
 };
 
 static unsigned int sizeof_process_field[process_field_count] = {
@@ -94,13 +94,6 @@ static void alloc_device_window(unsigned int start_row, unsigned int start_col,
                  sizeof_device_field[device_temperature] + sizeof_device_field[device_fan_speed]);
   if (dwin->power_info == NULL)
     goto alloc_error;
-  dwin->effective_load =
-      newwin(1, sizeof_device_field[device_effective_load], start_row + 1,
-             start_col + spacer * 5 + sizeof_device_field[device_clock] + sizeof_device_field[device_mem_clock] +
-                 sizeof_device_field[device_temperature] + sizeof_device_field[device_fan_speed] +
-                 sizeof_device_field[device_power]);
-  if (dwin->effective_load == NULL)
-    goto alloc_error;
 
   // Line 3 = GPU used | MEM used | Encoder | Decoder
 
@@ -206,7 +199,6 @@ static void free_device_windows(struct device_window *dwin) {
   delwin(dwin->gpu_clock_info);
   delwin(dwin->mem_clock_info);
   delwin(dwin->power_info);
-  delwin(dwin->effective_load);
   delwin(dwin->temperature);
   delwin(dwin->fan_speed);
   delwin(dwin->pcie_info);
@@ -353,7 +345,7 @@ static unsigned device_length(void) {
   return max(sizeof_device_field[device_name] + sizeof_device_field[device_pcie] + 1,
              sizeof_device_field[device_clock] + sizeof_device_field[device_mem_clock] +
                  sizeof_device_field[device_temperature] + sizeof_device_field[device_fan_speed] +
-                 sizeof_device_field[device_power] + sizeof_device_field[device_effective_load] + 5);
+                 sizeof_device_field[device_power] + 5);
 }
 
 static pid_t nvtop_pid;
@@ -501,6 +493,23 @@ static void draw_percentage_meter(WINDOW *win, const char *prelude, unsigned int
   wnoutrefresh(win);
 }
 
+// Draw percentage with a yellow highlight percentage (yellow percentage <= new_percentage)
+static void draw_percentage_meter_with_yellow_highlight(WINDOW *win, const char *prelude, unsigned int new_percentage,
+                                                        unsigned int yellow_percentage,
+                                                        const char inside_braces_right[1024]) {
+  draw_percentage_meter(win, prelude, new_percentage, inside_braces_right);
+  if (yellow_percentage > new_percentage)
+    yellow_percentage = new_percentage;
+  int rows, cols;
+  getmaxyx(win, rows, cols);
+  (void)rows;
+  size_t size_prelude = strlen(prelude);
+  int between_sbraces = cols - size_prelude - 2;
+  float usage = round((float)between_sbraces * yellow_percentage / 100.f);
+  mvwchgat(win, 0, size_prelude + 1, (int)usage, 0, yellow_color, NULL);
+  wnoutrefresh(win);
+}
+
 static const char *memory_prefix[] = {" B", "Ki", "Mi", "Gi", "Ti", "Pi"};
 
 static void draw_temp_color(WINDOW *win, unsigned int temp, unsigned int temp_slowdown, bool celsius) {
@@ -693,8 +702,15 @@ static void draw_devices(struct list_head *devices, struct nvtop_interface *inte
         draw_percentage_meter(decode_win, "DEC", rate, buff);
     }
     if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, gpu_util_rate)) {
-      snprintf(buff, 1024, "%u%%", device->dynamic_info.gpu_util_rate);
-      draw_percentage_meter(gpu_util_win, "GPU", device->dynamic_info.gpu_util_rate, buff);
+      if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, effective_load_rate)) {
+        snprintf(buff, 1024, "%u%%(eff %u%%)", device->dynamic_info.gpu_util_rate,
+                 device->dynamic_info.effective_load_rate);
+        draw_percentage_meter_with_yellow_highlight(gpu_util_win, "GPU", device->dynamic_info.gpu_util_rate,
+                                                    device->dynamic_info.effective_load_rate, buff);
+      } else {
+        snprintf(buff, 1024, "%u%%", device->dynamic_info.gpu_util_rate);
+        draw_percentage_meter(gpu_util_win, "GPU", device->dynamic_info.gpu_util_rate, buff);
+      }
     } else {
       snprintf(buff, 1024, "N/A");
       draw_percentage_meter(gpu_util_win, "GPU", 0, buff);
@@ -797,15 +813,6 @@ static void draw_devices(struct list_head *devices, struct nvtop_interface *inte
     mvwchgat(dev->power_info, 0, 0, 3, 0, cyan_color, NULL);
     wnoutrefresh(dev->power_info);
 
-    // EFFECTIVE LOAD
-    werase(dev->effective_load);
-    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, effective_load_rate))
-      mvwprintw(dev->effective_load, 0, 0, "Eff. %3u%%", device->dynamic_info.effective_load_rate);
-    else
-      mvwprintw(dev->effective_load, 0, 0, "Eff. N/A%%");
-    mvwchgat(dev->effective_load, 0, 0, 4, 0, cyan_color, NULL);
-    wnoutrefresh(dev->effective_load);
-
     // PICe throughput
     werase(dev->pcie_info);
     if (device->static_info.integrated_graphics) {
@@ -2088,14 +2095,14 @@ bool show_information_messages(unsigned num_messages, const char **messages) {
 }
 
 void print_snapshot(struct list_head *devices, bool use_fahrenheit_option) {
-  gpuinfo_populate_static_infos(devices);
-  gpuinfo_refresh_dynamic_info(devices);
   struct gpu_info *device;
 
   printf("[\n");
   list_for_each_entry(device, devices, list) {
     const char *indent_level_two = "  ";
     const char *indent_level_four = "   ";
+    const char *indent_level_six = "     ";
+    const char *indent_level_eight = "       ";
 
     const char *device_name_field = "device_name";
     const char *gpu_clock_field = "gpu_clock";
@@ -2166,26 +2173,162 @@ void print_snapshot(struct list_head *devices, bool use_fahrenheit_option) {
     else
       printf("%s\"%s\": null,\n", indent_level_four, gpu_util_field);
 
+    // Encode / Decode
+    if (device->static_info.encode_decode_shared) {
+      printf("%s\"encode_decode\": ", indent_level_four);
+      if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, decoder_rate))
+        printf("\"%u%%\",\n", device->dynamic_info.decoder_rate);
+      else
+        printf("null,\n");
+    } else {
+      printf("%s\"encode\": ", indent_level_four);
+      if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, encoder_rate))
+        printf("\"%u%%\",\n", device->dynamic_info.encoder_rate);
+      else
+        printf("null,\n");
+      printf("%s\"decode\": ", indent_level_four);
+      if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, decoder_rate))
+        printf("\"%u%%\",\n", device->dynamic_info.decoder_rate);
+      else
+        printf("null,\n");
+    }
+
     // Memory Utilization
     if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, mem_util_rate))
-      printf("%s\"%s\": \"%u%%\"\n", indent_level_four, mem_util_field, device->dynamic_info.mem_util_rate);
+      printf("%s\"%s\": \"%u%%\",\n", indent_level_four, mem_util_field, device->dynamic_info.mem_util_rate);
     else
-      printf("%s\"%s\": null\n", indent_level_four, mem_util_field);
+      printf("%s\"%s\": null,\n", indent_level_four, mem_util_field);
     // Memory Total
     if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, total_memory))
-      printf("%s\"%s\": \"%llu\"\n", indent_level_four, mem_total_field, device->dynamic_info.total_memory);
+      printf("%s\"%s\": \"%llu\",\n", indent_level_four, mem_total_field, device->dynamic_info.total_memory);
     else
-      printf("%s\"%s\": null\n", indent_level_four, mem_total_field);
+      printf("%s\"%s\": null,\n", indent_level_four, mem_total_field);
     // Memory Used
     if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, used_memory))
-      printf("%s\"%s\": \"%llu\"\n", indent_level_four, mem_used_field, device->dynamic_info.used_memory);
+      printf("%s\"%s\": \"%llu\",\n", indent_level_four, mem_used_field, device->dynamic_info.used_memory);
     else
-      printf("%s\"%s\": null\n", indent_level_four, mem_used_field);
+      printf("%s\"%s\": null,\n", indent_level_four, mem_used_field);
     // Memory Available
     if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, free_memory))
-      printf("%s\"%s\": \"%llu\"\n", indent_level_four, mem_free_field, device->dynamic_info.free_memory);
+      printf("%s\"%s\": \"%llu\",\n", indent_level_four, mem_free_field, device->dynamic_info.free_memory);
     else
-      printf("%s\"%s\": null\n", indent_level_four, mem_free_field);
+      printf("%s\"%s\": null,\n", indent_level_four, mem_free_field);
+
+    // Processes
+    printf("%s\"processes\" : [\n", indent_level_four);
+    for (unsigned i = 0; i < device->processes_count; ++i) {
+      struct gpu_process *proc = &device->processes[i];
+      printf("%s{\n", indent_level_six);
+
+      // PID
+      printf("%s\"pid\": \"%d\",\n", indent_level_eight, proc->pid);
+
+      printf("%s\"cmdline\": \"", indent_level_eight);
+      for (char *li = proc->cmdline; *li != '\0'; li++) {
+        // We need to escape some characters for for json strings
+        if (*li == '\n') {
+          printf("\\n");
+          continue;
+        } else if (*li == '\b') {
+          printf("\\b");
+          continue;
+        } else if (*li == '\f') {
+          printf("\\f");
+          continue;
+        } else if (*li == '\r') {
+          printf("\\r");
+          continue;
+        } else if (*li == '\t') {
+          printf("\\t");
+          continue;
+        }
+        // escaping backslash and quotes
+        if (*li == '\\' || *li == '"')
+          printf("\\");
+        printf("%c", *li);
+      }
+      printf("\",\n");
+
+      printf("%s\"kind\": ", indent_level_eight);
+      if (proc->type != gpu_process_unknown) {
+        printf("\"");
+        switch (proc->type) {
+        case gpu_process_graphical:
+          printf("graphic");
+          break;
+        case gpu_process_compute:
+          printf("compute");
+          break;
+        case gpu_process_graphical_compute:
+          printf("graphic & compute");
+          break;
+        default:
+          printf("N/A");
+          break;
+        }
+        printf("\"");
+      } else {
+        printf("null");
+      }
+      printf(",\n");
+
+      // GPU memory usage
+      printf("%s\"user\": ", indent_level_eight);
+      if (GPUINFO_PROCESS_FIELD_VALID(proc, user_name))
+        printf("\"%s\",\n", proc->user_name);
+      else
+        printf("null,\n");
+
+      // GPU usage
+      printf("%s\"gpu_usage\": ", indent_level_eight);
+      if (GPUINFO_PROCESS_FIELD_VALID(proc, gpu_usage))
+        printf("\"%u%%\",\n", proc->gpu_usage);
+      else
+        printf("null,\n");
+
+      // GPU memory usage
+      printf("%s\"gpu_mem_bytes_alloc\": ", indent_level_eight);
+      if (GPUINFO_PROCESS_FIELD_VALID(proc, gpu_memory_usage))
+        printf("\"%llu\",\n", proc->gpu_memory_usage);
+      else
+        printf("null,\n");
+
+      // GPU memory usage
+      printf("%s\"gpu_mem_usage\": ", indent_level_eight);
+      if (GPUINFO_PROCESS_FIELD_VALID(proc, gpu_memory_percentage))
+        printf("\"%u%%\",\n", proc->gpu_memory_percentage);
+      else
+        printf("null,\n");
+
+      // Encode usage
+      if (device->static_info.encode_decode_shared) {
+        // (Notice: no comma at the end as it's the last field here)
+        printf("%s\"encode_decode\": ", indent_level_eight);
+        if (GPUINFO_PROCESS_FIELD_VALID(proc, decode_usage))
+          printf("\"%u%%\"\n", proc->decode_usage);
+        else
+          printf("null\n");
+      } else {
+        printf("%s\"encode\": ", indent_level_eight);
+        if (GPUINFO_PROCESS_FIELD_VALID(proc, encode_usage))
+          printf("\"%u%%\",\n", proc->encode_usage);
+        else
+          printf("null,\n");
+        // (Notice: no comma at the end as it's the last field here)
+        printf("%s\"decode\": ", indent_level_eight);
+        if (GPUINFO_PROCESS_FIELD_VALID(proc, decode_usage))
+          printf("\"%u%%\"\n", proc->decode_usage);
+        else
+          printf("null\n");
+      }
+
+      printf("%s}", indent_level_six);
+      if (i != device->processes_count - 1)
+        printf(",");
+      printf("\n");
+    }
+    // (Notice: no comma at the end as it's the last field here)
+    printf("%s]\n", indent_level_four);
 
     if (device->list.next == devices)
       printf("%s}\n", indent_level_two);
@@ -2193,4 +2336,5 @@ void print_snapshot(struct list_head *devices, bool use_fahrenheit_option) {
       printf("%s},\n", indent_level_two);
   }
   printf("]\n");
+  fflush(stdout);
 }
